﻿using System;
using System.Collections.Generic;
using System.Linq;
using AbilityParser.Cost;
using AbilityParser.Effect;
using Magic.Utils;

namespace AbilityParser.Ability
{
	public class ActivatedAbility : IAbility
	{
		internal ICost Costs;
		internal IAbility Ability;

		public override string ToString()
		{
			string condition = Costs.ToString();
			string effect = Ability.ToString();
			return string.Format( "({0}) : ({1})", condition, effect );
		}

		public bool IsParsed()
		{
			return Costs.IsParsed() && Ability.IsParsed();
		}

		public IEnumerable<UnknownPart> GetUnknownParts()
		{
			List<UnknownPart> unknowns = new List<UnknownPart>();
			unknowns.AddRange(Costs.GetUnknownParts());
			unknowns.AddRange(Ability.GetUnknownParts());
			return unknowns;
		}

		public bool IsProxy()
		{
			return Costs.IsProxy() || Ability.IsProxy();
		}

		public IAbility RemoveProxy()
		{
			Costs = Costs.RemoveProxy();
			Ability = Ability.RemoveProxy();
			return this;
		}

		public AbContext SubParts
		{
			get { return Ability.SubParts; }
			set { Ability.SubParts = value; }
		}

		public static readonly ActivatedAbilityParser Parser = new ActivatedAbilityParser();

		public class ActivatedAbilityParser : Parser<IAbility>
		{
			public override bool Can( string text )
			{
				// 1. Colon-space
				if( text.IndexOf( CommonTokens.ColonSpace ) >= 0 ) return true;

				// 2. "Equip {3}"
				string parameterlessKw = CommonTokens.KwAbilitiesWithCost.FirstOrDefault( text.StartsWith );
				if ( IndexOfCostNoParams( parameterlessKw, text ) >= 0 ) return true;

				// 3. "Suspend 3 -- {U}"
				if ( text.StartsWith(CommonTokens.ReinforceToken) && text[CommonTokens.ReinforceToken.Length] == ' ' ) return true;
				if ( text.StartsWith(CommonTokens.SuspendToken) && text[CommonTokens.SuspendToken.Length] == ' ' ) return true;
				return false;
			}

			protected override IAbility Parse( string text )
			{
				ActivatedAbility result = new ActivatedAbility();
				string cost;
				string effect = ExtractCost( text, out cost );
				if (null == cost) { throw new InvalidOperationException("Activated ability parser found no cost"); }

				result.Costs = new UnparsedCost( cost );
				result.Ability = new UnparsedAbility( effect );
				return result;
			}

			protected static string ExtractCost(string source, out string myCost)
			{
				string[] parts = source.Split(CommonTokens.ActivationToken, StringSplitOptions.None);
				if (parts.Length > 1)
				{
					myCost = parts[0];
					return parts[1];
				}
				string parameterlessKw = CommonTokens.KwAbilitiesWithCost.FirstOrDefault(source.StartsWith);
				if (!String.IsNullOrWhiteSpace(parameterlessKw))
				{
					int idxCost = IndexOfCostNoParams( parameterlessKw, source );
					if ( idxCost >= 0 )
					{
						myCost = source.Substring( idxCost );
						return parameterlessKw;
					}
				}
				if (source.StartsWith(CommonTokens.ReinforceToken) && source[CommonTokens.ReinforceToken.Length] == ' ')
				{
					string[] others = source.Substring(CommonTokens.ReinforceToken.Length).Split(CommonTokens.EmDashSpace, 2, StringSplitOptions.RemoveEmptyEntries);
					myCost = String.Format("{0}, Discard this card", others[1]);
					return String.Format("{0} {1}", CommonTokens.ReinforceToken, others[0]).Trim();
				}
				if (source.StartsWith(CommonTokens.SuspendToken) && source[CommonTokens.SuspendToken.Length] == ' ')
				{
					string[] others = source.Substring(CommonTokens.SuspendToken.Length).Split(CommonTokens.EmDashSpace, 2, StringSplitOptions.RemoveEmptyEntries);
					myCost = others[1];
					return String.Format("{0} {1}", CommonTokens.SuspendToken, others[0]).Trim();
				}

				myCost = null;
				return source;

			}

			public static int IndexOfCostNoParams(string parameterlessKw, string source)
			{
				if (String.IsNullOrWhiteSpace(parameterlessKw)) return -1;
				int iChar = parameterlessKw.Length;
				char nextChar = source[iChar];
				bool hasSpaceAfterKeyword = nextChar == CommonTokens.EmDash || nextChar == ' ';
				if ( !hasSpaceAfterKeyword ) return -1;

				while(nextChar == CommonTokens.EmDash || nextChar == ' ')
				{
					iChar++;
					if (iChar >= source.Length) return -1;
					nextChar = source[iChar];
				}

				return Char.IsLower( nextChar ) ? -1 : iChar;
			}
		}
	}
}
