﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Magic.Utils;

namespace AbilityParser.Cost
{
	class CostComposite : ICost
	{
		internal IList<ICost> Costs = new List<ICost>();
		internal bool IsDisjunction;

		public bool IsParsed()
		{
			return Costs.All( c => c.IsParsed() );
		}

		public IEnumerable<UnknownPart> GetUnknownParts()
		{
			return Costs.SelectMany( c => c.GetUnknownParts() );
		}

		public bool IsProxy()
		{
			return Costs.Any(c => c.IsProxy());
		}

		public ICost RemoveProxy()
		{
			if( Costs.Count == 1 )
			{
				ICost willReturn = Costs[0];
				return willReturn.IsProxy() ? willReturn.RemoveProxy() : willReturn;
			}

			for( int i = 0; i < Costs.Count; i++ )
			{
				if (Costs[i].IsProxy())
					Costs[i] = Costs[i].RemoveProxy();
			}
			return this;
		}

		public override string  ToString()
		{
			return String.Join( IsDisjunction ? " -OR- " : ", ", Costs.Select( c => c.ToString() ) );
		}

		public static readonly Parser<ICost> ParseConjunction = new CompositeCostParserConjuction();
		public static readonly Parser<ICost> ParseDisjunction = new CompositeCostParserDisjuction();

		class CompositeCostParserDisjuction : Parser<ICost>
		{
			private const string OrBrace = " " + CommonTokens.Or + " {";
			private const string AndOrBrace = " " + CommonTokens.AndOr + " {";

			public override bool Can( string text )
			{
				return text.IndexOf( OrBrace ) > 0 || text.IndexOf( AndOrBrace ) > 0;
			}

			protected override ICost Parse( string text )
			{
				CostComposite result = new CostComposite();
				result.IsDisjunction = true;

				string token = AndOrBrace;
				int tokenPos = text.IndexOf( token );
				if (tokenPos == -1)
				{
					token = OrBrace;
					tokenPos = text.IndexOf( token );
				}
				result.Costs.Add( new UnparsedCost( text.Substring( 0, tokenPos ) ) );
				result.Costs.Add( new UnparsedCost( text.Substring( tokenPos  + token.Length - 1  ) ) );
				return result;
			}
		}

		class CompositeCostParserConjuction : Parser<ICost>
		{
			private static int IndexOfSeparator(string text, int startFrom = 0)
			{
				int nextCs = startFrom;
				int lastCs = nextCs;
				while (nextCs >= 0)
				{
					nextCs = text.IndexOf( CommonTokens.CommaSpace, lastCs );
					lastCs = nextCs + CommonTokens.CommaSpace.Length;
					if (lastCs < text.Length && !char.IsLower(text[lastCs]))
						return nextCs;
				}
				return -1;
			}

			public override bool Can( string text )
			{
				return IndexOfSeparator(text) >= 0;
			}

			protected override ICost Parse( string text )
			{
				CostComposite result = new CostComposite();
				int prevPart = 0;
				int nextPart = IndexOfSeparator(text, prevPart); ;
				while( nextPart != -1 )
				{
					result.Costs.Add( new UnparsedCost( text.Substring( prevPart, nextPart - prevPart ) ) );
					prevPart = nextPart + CommonTokens.CommaSpace.Length;
					nextPart = IndexOfSeparator(text, prevPart);
				}
				result.Costs.Add( new UnparsedCost( text.Substring( prevPart ) ) );
				return result;
			}
		}
	}


}
