﻿using System;
using Magic.Common;

namespace Magic.Cards
{
	public class CardColor : IComparable<CardColor>
	{
		private readonly byte myColor;
		private readonly int orderWeight;

		public CardColor(byte color) {
				myColor = color;
				orderWeight = OrderWeight; 
		}

		private CardColor() { myColor = 0; orderWeight = -1;}
		public readonly static CardColor Colorless = new CardColor();

		public bool HasAnyColor(byte colorMask) { return (myColor & colorMask) != 0; }
		public bool HasAllColors(byte colorMask) { return (myColor & colorMask) == colorMask; }

		public int CountColors() { int c = 0; for (byte v = myColor; v != 0; c++) { v &= (byte)(v - 1); } return c; } // bit count
		// order Has to be: W U B R G multi colorless - same as cards numbering through a set
		public int OrderWeight { get { return myColor == 0 ? 0x400 : (CountColors() == 1 ? myColor : 0x200); }}

		public bool IsColorless() { return myColor == 0; }
		public bool IsMulticolor() { return CountColors() > 1; }
		public bool IsMonoColor() { return CountColors() == 1; }

		public bool Equals(byte color) { return color == myColor; }


		public int CompareTo(CardColor other) { return other != null ? orderWeight - other.orderWeight : 1; }

		// Presets
		public bool HasWhite() { return HasAnyColor(MagicColor.White); }
		public bool HasBlue() { return HasAnyColor(MagicColor.Blue); }
		public bool HasBlack() { return HasAnyColor(MagicColor.Black); }
		public bool HasRed() { return HasAnyColor(MagicColor.Red); }
		public bool HasGreen() { return HasAnyColor(MagicColor.Green); }

		public bool IsWhite() { return Equals(MagicColor.White); }
		public bool IsBlue() { return Equals(MagicColor.Blue); }
		public bool IsBlack() { return Equals(MagicColor.Black); }
		public bool IsRed() { return Equals(MagicColor.Red); }
		public bool IsGreen() { return Equals(MagicColor.Green); }

		override public String ToString() {
			if (orderWeight == -1) return "n/a";
			switch (myColor) {
				case 0: return "colorless";
				case MagicColor.White: return "white";
				case MagicColor.Blue: return "blue";
				case MagicColor.Black: return "black";
				case MagicColor.Red: return "red";
				case MagicColor.Green: return "green";
				default: return "multi";
			}
		}

		public static bool operator <(CardColor set1, CardColor set2) { return set1 != null ? set1.CompareTo(set2) < 0 : true; }
		public static bool operator >(CardColor set1, CardColor set2) { return set1 != null ? set1.CompareTo(set2) > 0 : false; }
		public static bool operator <=(CardColor set1, CardColor set2) { return set1 != null ? set1.CompareTo(set2) <= 0 : true; }
		public static bool operator >=(CardColor set1, CardColor set2) { return set1 != null ? set1.CompareTo(set2) >= 0 : false; }
		public static bool operator ==(CardColor set1, CardColor set2) { return set1 != null ? set1.Equals(set2) : false; }
		public static bool operator !=(CardColor set1, CardColor set2) { return set1 != null ? !set1.Equals(set2) : false; }

		public override bool Equals(object obj)
		{
			if( ReferenceEquals( null, obj ) ) return false;
			return ReferenceEquals( this, obj ) || Equals( obj as CardColor );
		}

		public bool Equals( CardColor other )
		{
			if( ReferenceEquals( null, other ) ) return false;
			if( ReferenceEquals( this, other ) ) return true;
			return other.myColor == myColor;
		}

		public override int GetHashCode()
		{
			unchecked
			{
				return ( myColor.GetHashCode()*397 ) ^ orderWeight;
			}
		}
	}
}