﻿using System;
using System.Collections.Generic;
using System.Diagnostics.Contracts;
using System.Linq;

namespace Magic.Cards
{
	public class CardType : IComparable<CardType>
	{
		private readonly ISet<String> subType = new SortedSet<String>();
		private readonly ISet<CardTypeCore> coreType = new SortedSet<CardTypeCore>();
		private readonly ISet<CardTypeSuper> superType = new SortedSet<CardTypeSuper>();
		private String calculatedType; // since obj is immutable, this Is calc'd once

		private CardType() { } // use static ctors!

		public static CardType Parse(Tuple<String,String>[] types)
		{
			if (null == types || types.Length == 0) return null;
			CardType result = new CardType();
			foreach( Tuple<string, string> type in types )
			{
				CardTypeCore ct;
				if (type.Item1 == "card" && Enum.TryParse(type.Item2, true, out ct)) result.coreType.Add( ct );
				CardTypeSuper st;
				if (type.Item1 == "super" && Enum.TryParse(type.Item2, true, out st)) result.superType.Add(st);
				if (type.Item1 == "sub") result.subType.Add( type.Item2 );
			}
			return result;
		}

		// TODO: Debug this code
		public static CardType Parse(String typeText) {
			Contract.Requires(typeText != null);
			if (null == typeText) return null;

			// Most types and subtypes, except "Serra's Realm" and "Bolas's Meditation Realm" consist of only one word
			const char space = ' ';
			CardType result = new CardType();

			int iTypeStart = 0;
			int iSpace = typeText.IndexOf(space);
			bool hasMoreTypes = typeText.Length > 0;
			while (hasMoreTypes) {
				String type = typeText.Substring(iTypeStart, iSpace == -1 ? typeText.Length : iSpace );
				hasMoreTypes = iSpace != -1;
				if (!IsMultiwordType(type) || !hasMoreTypes) {
						iTypeStart = iSpace + 1;
						result.parseAndAdd(type);
				}
				iSpace = typeText.IndexOf(space, iSpace + 1);
			}
			return result;
		}

		static readonly String[] multiWordTypes = new[] {"Serra's Realm", "Bolas's Meditation Realm"};
		private static bool IsMultiwordType(String type) {
				
			// no need to loop for only 2 exceptions!
			if (multiWordTypes[0].StartsWith(type) && multiWordTypes[0] != type) { return true; }
			if (multiWordTypes[1].StartsWith(type) && multiWordTypes[1] != type) { return true; }
			return false;
		}

		private void parseAndAdd(String type) {
				if ("-" == type) { return; }

				CardTypeCore ct;
				if (Enum.TryParse(type, out ct)) { coreType.Add(ct); return; }

				CardTypeSuper st;
				if (Enum.TryParse(type, out st)) { superType.Add(st); return; }

				// If not recognized by super- and core- this must be subtype
				subType.Add(type);
		}

		public bool SubTypeContains(String operand) { return subType.Contains(operand); }
		public bool TypeContains(CardTypeCore operand) { return coreType.Contains(operand); }
		public bool SuperTypeContains(CardTypeSuper operand) { return superType.Contains(operand); }

		public bool Is(CardTypeCore type) { return coreType.Contains(type); }

		public bool IsBasic() { return superType.Contains(CardTypeSuper.Basic); }
		public bool IsLegendary() { return superType.Contains(CardTypeSuper.Legendary); }
		public bool IsBasicLand() { return IsBasic() && Is(CardTypeCore.Land); }

		public String TypesBeforeDash { get {
				IList<String> types = new List<String>();
				foreach (CardTypeSuper st in superType) { types.Add(st.ToString()); }
				foreach (CardTypeCore ct in coreType) { types.Add(ct.ToString()); }
				return String.Join(" ", types);
		} }

		public String TypesAfterDash { get { return String.Join(" ", subType); } }

		public override String ToString()
		{
			return calculatedType ?? ( calculatedType = toStringImpl() );
		}

		private String toStringImpl()
		{
			return subType.Any() ? String.Format("{0} - {1}", TypesBeforeDash, TypesAfterDash) : TypesBeforeDash;
		}

		public int CompareTo(CardType other) {
			return other == null ? 1 : ToString().CompareTo(other.ToString());
		}
		
}
}
