﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using AbilityParser;
using Magic.Cards;
using Magic.Utils;

namespace MtgData
{
	[CLSCompliant(true)]
	public class CardDatabase
	{
		public IDictionary<string, CardSet> Sets { get; private set; }
		public IDictionary<string, CardRules> Cards { get; private set; }
		public IEnumerable<CardPrinted> Prints { get; private set; }

		private static volatile CardDatabase onlyInstance = null;
		
		private static readonly Object syncRoot = new Object();
		public static CardDatabase Instance() {
			if (onlyInstance == null) {
				throw new InvalidOperationException("Card database has not yet been initialized, run setup() first");
			}
			return onlyInstance;
		}

		public static void Setup(String pathToXml, String pathToUnclear)
		{
			if (onlyInstance != null) {
				throw new InvalidOperationException("Card database has already been initialized, don't do it twice please");
			}
			lock (syncRoot) {
				if (onlyInstance == null) { // It's broken under 1.4 and below, on 1.5+ works again!
					onlyInstance = new CardDatabase(pathToXml, pathToUnclear);
				}
			}
		}

		private CardDatabase( String pathToXml, String pathToSaveUnclearParts)
		{
			string[] unwantedSets = new[] { "VGO", "VG", "VG1", "VG2", "VG3", "VG4", "ASTRAL", "SDC", "UGL", "UNH", "MED", "ME2", "ME3", "ME4", "ATH", "HOP" };
			ReaderCards reader = new ReaderCards( unwantedSets );
			Sets = ReadOnlyDictionary<string, CardSet>.From( reader.ReadSets( Path.Combine( pathToXml, "setinfo.xml" ) ).ToDictionary( s => s.Code ) );
			IDictionary<string, CardRules> dictCards = reader.ReadRules( Path.Combine( pathToXml, "cards.xml" ) ).ToDictionary( r => r.Name );
			Cards = ReadOnlyDictionary<string, CardRules>.From( dictCards );
			Prints = reader.ReadPrints(Path.Combine(pathToXml, "meta.xml"));

			foreach( CardPrinted print in Prints )
			{
				CardRules rules;
				if( !Cards.TryGetValue( print.Name, out rules ) )
					throw new KeyNotFoundException( "Card rules not found for: " + print.Name );
				
				print.Rules = rules;
				rules.RegisterPrinting( print.Set, print.Rarity );
			}

			// Remove unlinked cards
			var cc = Cards.Values.Where( c => c.PrintedIn.Count == 0 ).Select( c => c.Name ).ToArray();
			foreach( string s in cc )
			{
				dictCards.Remove( s );
			}

			// Parse!
			foreach (CardRules rules in Cards.Values)
			{
				RootParser.Parse(rules);
			}

			//DumpRawTexts("D:/temp/abilities.txt");
			DumpUnclearAbilities( Path.Combine( pathToSaveUnclearParts, "unclear.txt" ) ); 

		}

		public void DumpUnclearAbilities(string filename)
		{
			List<Tuple<string, UnknownPart>> allLines = new List<Tuple<string, UnknownPart>>();
			foreach (CardRules rule in Cards.Values)
			{
				CardRulesInner rule1 = rule;
				allLines.AddRange(rule1.Abilities.SelectMany(a => a.GetUnknownParts()).Select( up => Tuple.Create( rule1.Name, up)));
				if (rule.secondPart == null) continue;
				CardRulesInner rule2 = rule.secondPart;
				allLines.AddRange(rule2.Abilities.SelectMany(a => a.GetUnknownParts()).Select(up => Tuple.Create( rule2.Name, up)));
			}

			var data = allLines.GroupBy( tt => tt.Item2, ( pp, eTpl ) => Tuple.Create( pp, eTpl.Select( t => t.Item1 ).ToArray() ) );
			var toFile = data.Select(ParseProblemGroupToString);
			File.WriteAllLines(filename, toFile);
		}

		private static string ParseProblemGroupToString(Tuple<UnknownPart, string[]> d )
		{
			string cards = String.Join( "; ", d.Item2.Take( 10 ) );
			return String.Format( "{0}|{1}|{2}|{3}|{4}", d.Item2.Length, d.Item1.TextType, d.Item1.Text, d.Item1.ParserClass.Name, cards );
		}

		public void DumpRawTexts(string filename)
		{
			List<string> allLines = new List<string>();
			foreach (CardRules rule in Cards.Values)
			{
				CardRulesInner rule1 = rule;
				allLines.AddRange(rule1.Texts.Select(t => String.Format("{0}|{1}", rule1.Name, t)));
				if (rule.secondPart == null) continue;
				CardRulesInner rule2 = rule.secondPart;
				allLines.AddRange(rule2.Texts.Select(t => String.Format("{0}|{1}", rule2.Name, t)));
			}
			File.WriteAllLines(filename, allLines);
		}


	}

}
