﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Xml.Linq;
using Magic.Cards;
using Magic.Common;
using Magic.Mana;

namespace MtgData
{
	public class ReaderCards
	{
		protected string[] ExcludeSets;
		public ReaderCards(string[] excludeSets)
		{
			ExcludeSets = excludeSets;
		}

		public ReadOnlyCollection<CardPrinted> ReadPrints( string fileName )
		{
			List<CardPrinted> result = new List<CardPrinted>();
			XDocument xDoc = XDocument.Load(fileName);
			if (xDoc.Root != null)
			{
				var elts = xDoc.Root.Elements();

				foreach (var r in elts.Select(ReadSinglePrint).Where(s => s != null))
					result.AddRange(r);
			}
			return new ReadOnlyCollection<CardPrinted>(result);
		}

		public IEnumerable<CardPrinted> ReadSinglePrint(XElement xNode)
		{
			XElement nameEl = xNode.Element("name");
			if (null == nameEl)
				throw new Exception("Name tag missing in meta description");

			string lastSet = null;
			int artIndex = 1;
			foreach (var xInst in xNode.Elements("instance"))
			{
				XElement xSet = xInst.Element( "set" );
				if( xSet == null || ExcludeSets.Contains( xSet.Value ) ) continue;

				String set = xSet.Value;
				if (set == lastSet) artIndex++; else { lastSet = set; artIndex = 1; }
				CardPrinted meta = new CardPrinted {Name = nameEl.Value, Rarity = CardRarity.NotApplicable, Set = set, ArtIndex = artIndex};

				XElement xRarity = xInst.Element( "rarity" );
				if ( xRarity != null ) switch( xRarity.Value ) {
					case "C": meta.Rarity = CardRarity.Common; break;
					case "U": meta.Rarity = CardRarity.Uncommon; break;
					case "R": meta.Rarity = CardRarity.Rare; break;
					case "M": meta.Rarity = CardRarity.MythicRare; break;
					case "S": meta.Rarity = CardRarity.Special; break;
					case "L": meta.Rarity = CardRarity.BasicLand; break;
				}
				
				XElement xFlavour = xInst.Element( "flavour-text" );
				meta.FlavorText = xFlavour == null ? null : xFlavour.Value;
				XElement xArtist = xInst.Element("artist");
				meta.Artist = xArtist == null ? null : xArtist.Value;

				yield return meta;
			}
		}

		public ReadOnlyCollection<CardSet> ReadSets(string fileName)
		{
			List<CardSet> result = new List<CardSet>();
			XDocument xDoc = XDocument.Load(fileName);
			if (xDoc.Root != null)
			{
				var elts = xDoc.Root.Elements();
				result.AddRange(elts.Select(ReadSingleSet).Where(s => s != null));
			}
			return new ReadOnlyCollection<CardSet>(result);
		}

		private CardSet ReadSingleSet(XElement xNode)
		{
			XElement nameEl = xNode.Element("name");
			if (null == nameEl)
				throw new Exception("Name tag missing in set description");
			String name = nameEl.Value;
			XElement xCode = xNode.Element("code");
			if (null == xCode || ExcludeSets.Contains(xCode.Value))
				return null;
			String code = xCode.Value;
			String block = xNode.Element("block") != null ? xNode.Element("block").Value : String.Empty;
			//if (xNode.Element("cards") == null || !int.TryParse(xNode.Element("cards").Value, out set.CardCount)) set.CardCount = 0;
			string sDate = xNode.Element("release-date") == null ? "1993-1-1" : xNode.Element("release-date").Value;
			int[] parts = sDate.Split(new[] { '-' }).Select(s => int.Parse(s)).ToArray();
			DateTime release = new DateTime(parts[0], parts.Length > 1 ? parts[1] : 1, parts.Length > 2 ? parts[2] : 1);
			return new CardSet(release, name, code, block);
		}

		public List<CardRules> ReadRules(string filename)
		{
			List<CardRules> result = new List<CardRules>();
			XDocument xDoc = XDocument.Load(filename);
			if (xDoc.Root != null)
			{
				var elts = xDoc.Root.Elements();
				result.AddRange(elts.Select(ReadSingleCard).Where(c => c != null));
			}
			return result;
		}

		public static CardRules ReadSingleCard(XElement cardNode)
		{
			XElement hand = cardNode.Element("hand");
			if (hand != null) return null;

			XElement secondFace = cardNode.Element("multi");
			bool isDoubleFaced = secondFace != null;

			CardRules card = new CardRules();
			ReadCardFace(cardNode, card);
			if (isDoubleFaced)
			{
				card.secondPart = ReadCardFace(secondFace);
			}
			return card;
		}

		static readonly string[] emptyArray = new string[] { };
		public static CardRulesInner ReadCardFace(XElement cardNode, CardRulesInner rules = null)
		{
			CardRulesInner card = rules ?? new CardRulesInner();

			XElement nameEl = cardNode.Element("name");
			if (null == nameEl)
				throw new Exception("Name tag missing in card description");
			card.Name = nameEl.Value;

			XElement costEl = cardNode.Element("cost");
			card.Cost = ManaCost.Parse(costEl == null ? null : costEl.Value);

			XElement typelistElt = cardNode.Element("typelist");
			if (null == typelistElt)
				throw new Exception("Card " + card.Name + " has no types");
			card.CardType = CardType.Parse(typelistElt.Elements().Select(xe => Tuple.Create(xe.Attribute("type").Value, xe.Value)).ToArray());

			XElement xColor = cardNode.Element("color");
			byte color = 0;
			if (xColor != null)
			{
				foreach (string sColor in xColor.Value.Split(new char[] { '/' }))
				{
					switch (sColor)
					{
						case "White": color |= MagicColor.White; break;
						case "Red": color |= MagicColor.Red; break;
						case "Blue": color |= MagicColor.Blue; break;
						case "Black": color |= MagicColor.Black; break;
						case "Green": color |= MagicColor.Green; break;
					}
				}
			}
			else
				color = card.Cost.ColorProfile;
			card.Color = new CardColor(color);

			var rulelistElt = cardNode.Element("rulelist");
			card.setTexts(null == rulelistElt ? emptyArray : rulelistElt.Elements().Select(xe => xe.Value).Where(s => !String.IsNullOrWhiteSpace(s)).ToArray());

			var loyaltyElt = cardNode.Element("loyalty");
			var powerElt = cardNode.Element("pow");
			var toughnessElt = cardNode.Element("tgh");
			card.Loyalty = loyaltyElt == null ? (short)0 : short.Parse(loyaltyElt.Value);
			card.Power = powerElt == null ? null : powerElt.Value;
			card.Toughness = toughnessElt == null ? null : toughnessElt.Value;

			return card;
		}
	}
}
