using System;
using System.Globalization;
using System.Windows.Data;
using System.Windows.Media;
using System.Linq;
using Magic.Cards;

namespace WpfRules
{
	public class CellColorFromParseStatus : IValueConverter
	{
		public object Convert( object value, Type targetType, object parameter, CultureInfo culture )
		{
			CardPrinted cp = value as CardPrinted;
			int nRules = cp.Rules.Abilities.Count;
			float nRulesParsed = cp.Rules.Abilities.Count( a => a.IsParsed() );
			float percent = nRules == 0 ? 1 : nRulesParsed / nRules;

			return new SolidColorBrush(colorForPercent(percent));
		}

		public object ConvertBack( object value, Type targetType, object parameter, CultureInfo culture )
		{
			throw new NotImplementedException();
		}

		private static Color colorForPercent( float value )
		{
			byte r, g, b;
			HsvToRgb(value * value * 120, 0.3 + value * 0.4, 1, out r, out g, out b);
			return Color.FromRgb( r, g, b );
		}

		/// <summary>
		/// Convert HSV to RGB
		/// h is from 0-360
		/// s,v values are 0-1
		/// r,g,b values are 0-255
		/// Based upon http://ilab.usc.edu/wiki/index.php/HSV_And_H2SV_Color_Space#HSV_Transformation_C_.2F_C.2B.2B_Code_2
		/// </summary>
		private static void HsvToRgb(double h, double S, double V, out byte r, out byte g, out byte b)
		{
			// ######################################################################
			// T. Nathan Mundhenk
			// mundhenk@usc.edu
			// C/C++ Macro HSV to RGB

			double H = h;
			while (H < 0) { H += 360; };
			while (H >= 360) { H -= 360; };
			double R, G, B;
			if (V <= 0)
			{ R = G = B = 0; }
			else if (S <= 0)
			{
				R = G = B = V;
			}
			else
			{
				double hf = H / 60.0;
				int i = (int)Math.Floor(hf);
				double f = hf - i;
				double pv = V * (1 - S);
				double qv = V * (1 - S * f);
				double tv = V * (1 - S * (1 - f));
				switch (i)
				{

					// Red is the dominant color

					case 0:
						R = V;
						G = tv;
						B = pv;
						break;

					// Green is the dominant color

					case 1:
						R = qv;
						G = V;
						B = pv;
						break;
					case 2:
						R = pv;
						G = V;
						B = tv;
						break;

					// Blue is the dominant color

					case 3:
						R = pv;
						G = qv;
						B = V;
						break;
					case 4:
						R = tv;
						G = pv;
						B = V;
						break;

					// Red is the dominant color

					case 5:
						R = V;
						G = pv;
						B = qv;
						break;

					// Just in case we overshoot on our math by a little, we put these here. Since its a switch it won't slow us down at all to put these here.

					case 6:
						R = V;
						G = tv;
						B = pv;
						break;
					case -1:
						R = V;
						G = pv;
						B = qv;
						break;

					// The color is not defined, we should throw an error.

					default:
						//LFATAL("i Value error in Pixel conversion, Value is %d", i);
						R = G = B = V; // Just pretend its black/white
						break;
				}
			}
			r = Clamp(R * 255.0);
			g = Clamp(G * 255.0);
			b = Clamp(B * 255.0);
		}

		/// <summary>
		/// Clamp a value to 0-255
		/// </summary>
		static byte Clamp(double i)
		{
			if (i < 0) return 0;
			if (i > 255) return 255;
			return (byte)i;
		}
	}
}