﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.IO;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media.Imaging;
using Magic.Cards;
using Magic.Utils;
using MtgData;

namespace WpfRules
{
	/// <summary>
	/// Interaction logic for MainWindow.xaml
	/// </summary>
	public partial class MainWindow : Window
	{

		protected CardDatabase model;


		private string _nameFilter;
		public string NameFilter { 
			get { return _nameFilter; } 
			set { _nameFilter = value; UpdateFilter(); }
		}

		private string _typeFilter;
		public string TypeFilter
		{
			get { return _typeFilter; }
			set { _typeFilter = value; UpdateFilter(); }
		}

		private string _rulesFilter;
		public string RulesFilter
		{
			get { return _rulesFilter; }
			set { _rulesFilter = value; UpdateFilter(); }
		}

		private bool _uniqFilter = true;
		public bool UniqueFilter
		{
			get { return _uniqFilter; }
			set { _uniqFilter = value; UpdateFilter(); }
		}
		
		
		private IList<CardPrinted> displayedCards;
		private IList<CardPrinted> filteredCards;

		protected string PathToImages = ConfigurationManager.AppSettings["PathToImages"];

		public MainWindow()
		{
			InitializeComponent();
			CardDatabase.Setup(ConfigurationManager.AppSettings["PathToCardsXml"], ConfigurationManager.AppSettings["PathToUnclear"]);
			model = CardDatabase.Instance();
			
			tvEditions.Items.Clear();
			
			foreach( TreeViewItem node in from set in model.Sets.OrderByDescending( s => s.Value.ReleasedOn )
			                           select new TreeViewItem{Header = set.Value.Name, Tag = set.Key} )
			{
				tvEditions.Items.Add( node );
			}

		}

		private void buttonAllSets_Click(object sender, RoutedEventArgs e)
		{
			DisplayCardList(model.Prints);
		}

		private void tvEditions_SelectedItemChanged(object sender, RoutedPropertyChangedEventArgs<object> e)
		{
			string set = ((TreeViewItem)e.NewValue).Tag as string;
			if (null == set || set.Contains("BLOCK")) return;
			DisplayCardList(model.Prints.Where(p => p.Set == set));
		}

		private void DisplayCardList(IEnumerable<CardPrinted> cards)
		{
			displayedCards = cards.ToArray();
			UpdateFilter();
		}

		private void UpdateFilter()
		{
			filteredCards = FilterCards(displayedCards);
			gridCards.ItemsSource = filteredCards;
		}

		private readonly CardComparerByRulesName uniqComparer = new CardComparerByRulesName();
		private IList<CardPrinted> FilterCards(IEnumerable<CardPrinted> source)
		{
			IEnumerable<CardPrinted> modSource = source;
			if (null == source) return null;

			if (_uniqFilter)
			{
				modSource = modSource.Distinct( uniqComparer );
			}
			if( !String.IsNullOrWhiteSpace( _nameFilter ) )
			{
				modSource = modSource.Where( c => c.Name.IndexOf(_nameFilter, StringComparison.OrdinalIgnoreCase) >= 0 );
			}

			if (null != _rulesFilter)
			//if (!String.IsNullOrWhiteSpace(_rulesFilter))
			{
				modSource = modSource.Where( c => c.Rules.Texts.Any(t => t.IndexOf(_rulesFilter, StringComparison.OrdinalIgnoreCase) >= 0) );
			}

			if (!String.IsNullOrWhiteSpace(_typeFilter))
			{
				modSource = modSource.Where( c => c.Rules.CardType.ToString().IndexOf( _typeFilter, StringComparison.OrdinalIgnoreCase) >= 0 );
			}
			
			return modSource.ToArray();
		}

		private void gridCards_SelectionChanged(object sender, SelectionChangedEventArgs e)
		{
			int idx = gridCards.SelectedIndex;
			if (idx < 0) return;

			CardPrinted cp = filteredCards[idx];
			DisplayCardAtRight(cp);
		}

		private void DisplayPitcure(CardPrinted card)
		{
			string setToUse = card.Set;
			string nameToUse = card.Rules.Name;
			if (card.Rules.secondPart != null)
			{
				nameToUse += card.Rules.secondPart.Name;
			}

			if (model.Cards[card.Name].PrintedIn[setToUse].Prints > 1)
				nameToUse += card.ArtIndex.ToString();
			nameToUse = nameToUse.Replace( ":", "" );

			if (setToUse == null) return;

			if (setToUse == "CON")
				setToUse = "CFX";
			string pathToImage = Path.Combine(PathToImages, setToUse, nameToUse + ".full.jpg");
			pathToImage = pathToImage.Replace("Æ", "Ae");
			try
			{
				picCard.Source = new BitmapImage(new Uri(pathToImage));
			}
			catch (IOException)
			{
				picCard.Source = null;
			}
		}

		private void DisplayCardAtRight(CardPrinted card)
		{
			DisplayPitcure(card);

			listRules.Items.Clear();
			if (card.Rules.Texts != null)
				foreach( string text in card.Rules.Texts )
				{
					listRules.Items.Add( text );
				}

			listRules.Items.Add( "==================" );

			foreach( IParseUnit<object> ability in card.Rules.Abilities )
			{
				listRules.Items.Add( ability.ToString() );
			}

		}




	}
}
